@@ -16,6 +16,7 @@ CONFIG_SCHEMA = {
     'inverse_colors': bool,
     'portrait': bool,
     'ui_page': int,
+    'recent_files': (dict, int),
 }
 
 def main():
@@ -30,12 +31,13 @@ def main():
         pass
 
     if MAEMO:
-        sdirs = [os.path.expanduser("~/MyDocs/.documents/Books")]
+        sdirs = [os.path.expanduser("~/MyDocs/Books")]
     else:
         sdirs = [os.path.join(os.path.expanduser("~"), "Desktop", "Books")]
     config.setdefault('search_dirs', sdirs)
     config.setdefault('save_dir', sdirs[0])
     config.setdefault('positions', {})
+    config.setdefault('recent_files', {})
     config.setdefault('inverse_colors', False)
     config.setdefault('portrait', False)
     config.setdefault('ui_page', 1)
@@ -52,7 +54,8 @@ class MGutenbergApp(AppBase):
         AppBase.__init__(self)
         
         self.config = config
-        self.ebook_list = EbookList(config['search_dirs'])
+        self.ebook_list = EbookList(config['search_dirs'],
+                                    config['recent_files'])
         self.window = MainWindow(self)
         self.readers = []
 
@@ -90,6 +93,7 @@ class MGutenbergApp(AppBase):
             return self.show_notify(widget, _("Working..."))
 
     def start_reader(self, filename):
+        self.ebook_list.mark_visited(filename, self.config['recent_files'])
         reader.run(self, filename)
 
     def run(self, args):
@@ -139,14 +143,17 @@ class MainWindow(object):
 
     LOCAL_PAGE = 0
     GUTENBERG_PAGE = 1
+    RECENT_PAGE = 2
 
     def __init__(self, app):
         self.app = app
 
         self.ebook_list = EbookListWidget(self.app)
+        self.recent_list = EbookListWidget(self.app,
+                                           self.app.ebook_list.recent_list)
         self.gutenberg_search = GutenbergSearchWidget(self.app)
 
-        self._start_ui_page = self.app.config['ui_page'] % 2
+        self._start_ui_page = self.app.config['ui_page'] % 3
 
         self._construct()
         self.widget.connect("destroy", self.on_destroy)
@@ -157,6 +164,7 @@ class MainWindow(object):
         self.widget.show_all()
         if MAEMO:
             self.ebook_list.search.hide()
+            self.recent_list.search.hide()
 
         # Set up menu
         if self.menu is not None:
@@ -220,6 +228,11 @@ class MainWindow(object):
 
         self.gutenberg_search.search_button.grab_default()
 
+        # Recent ebooks tab
+        page = gtk.VBox()
+        page.pack_end(self.recent_list.widget, expand=True, fill=True)
+        self.notebook.append_page(page, gtk.Label(_("Recent")))
+
         if MAEMO:
             # Tabs are changed from menu
             self.notebook.set_show_tabs(False)
@@ -270,6 +283,10 @@ class MainWindow(object):
         self.notebook.set_current_page(self.GUTENBERG_PAGE)
         self.on_notebook_switch_page(None, None, self.GUTENBERG_PAGE)
 
+    def on_recent_button_click(self, widget):
+        self.notebook.set_current_page(self.RECENT_PAGE)
+        self.on_notebook_switch_page(None, None, self.RECENT_PAGE)
+
     def on_notebook_switch_page(self, notebook, page, page_num):
         self.app.config['ui_page'] = int(page_num)
 
@@ -277,18 +294,25 @@ class MainWindow(object):
             self.book_button.handler_block_by_func(self.on_book_button_click)
             self.gutenberg_button.handler_block_by_func(
                 self.on_gutenberg_button_click)
+            self.recent_button.handler_block_by_func(
+                self.on_recent_button_click)
 
             self.book_button.set_active(page_num == self.LOCAL_PAGE)
             self.gutenberg_button.set_active(page_num == self.GUTENBERG_PAGE)
+            self.recent_button.set_active(page_num == self.RECENT_PAGE)
 
             if page_num == self.LOCAL_PAGE:
                 self.delete_file_button.show()
             elif page_num == self.GUTENBERG_PAGE:
                 self.delete_file_button.hide()
+            elif page_num == self.RECENT_PAGE:
+                self.delete_file_button.hide()
 
             self.book_button.handler_unblock_by_func(self.on_book_button_click)
             self.gutenberg_button.handler_unblock_by_func(
                 self.on_gutenberg_button_click)
+            self.recent_button.handler_unblock_by_func(
+                self.on_recent_button_click)
 
     def on_open_file(self, widget):
         m = hildon.FileSystemModel()
@@ -341,14 +365,17 @@ class MainWindow(object):
         menu = hildon.AppMenu()
 
         # Filter buttons
-        self.gutenberg_button = gtk.ToggleButton(label=_("Project Gutenberg"))
+        self.gutenberg_button = gtk.ToggleButton(label=_("Gutenberg"))
         self.book_button = gtk.ToggleButton(label=_("Local books"))
+        self.recent_button = gtk.ToggleButton(label=_("Recent"))
 
         self.gutenberg_button.connect("toggled", self.on_gutenberg_button_click)
         self.book_button.connect("toggled", self.on_book_button_click)
+        self.recent_button.connect("toggled", self.on_recent_button_click)
 
         menu.add_filter(self.gutenberg_button)
         menu.add_filter(self.book_button)
+        menu.add_filter(self.recent_button)
 
         open_file_button = gtk.Button(label=_("Open file"))
         open_file_button.connect("clicked", self.on_open_file)
@@ -376,11 +403,14 @@ class EbookListWidget(object):
     </ui>
     """]
 
-    def __init__(self, app):
+    def __init__(self, app, store=None):
         self.app = app
         self.filter_text = ''
 
-        self.store = app.ebook_list
+        if store is None:
+            self.store = app.ebook_list
+        else:
+            self.store = store
         self.filter = None
         self.filtered_store = None
         
@@ -484,8 +514,19 @@ class EbookListWidget(object):
     def on_menu_delete(self, widget):
         model = self.widget_tree.get_model()
         try:
-            it = model.get_iter(self._active_item)
-            entry = model[it]
+            pth = self._active_item
+            self._active_item = None
+
+            entry = model[model.get_iter(pth)]
+
+            # De-filter the view
+            while model is not None and model is not self.store:
+                pth = model.convert_path_to_child_path(pth)
+                model = model.get_model()
+            if model != self.store:
+                raise RuntimeError("Programming error -- please report")
+            it = model.get_iter(pth)
+
             self._active_item = None
 
             def response(widget, response_id):
@@ -897,7 +938,7 @@ class GutenbergDownloadWindow(object):
                     dlg.add_button(_("Read it"), 1)
                     dlg.add_button(gtk.STOCK_CANCEL, 0)
                     def response(dlg, response_id, path):
-                        if response_id:
+                        if response_id == 1:
                             self.app.start_reader(path)
                         dlg.destroy()
                     dlg.connect("response", response, path)
