@@ -19,7 +19,7 @@ Config
 
 """
 
-import re, os, sys, shutil, tempfile
+import re, os, sys, shutil, tempfile, time
 import xml.etree.ElementTree as ET
 from xml.parsers.expat import ExpatError
 
@@ -33,6 +33,10 @@ from util import *
 class OverwriteFileException(Exception): pass
 
 
+#------------------------------------------------------------------------------
+# Local book list
+#------------------------------------------------------------------------------
+
 def _is_caps_case(x):
     if len(x) < 1:
         return False
@@ -55,7 +59,7 @@ def get_valid_basename(base):
                  '.tcr',
                  '.tgz', '.ipk',
                  ]
-    skip_ext = ['.gz', '.bz2', '.tar']
+    skip_ext = ['.gz', '.bz2', '.tar', '.utf8', '.ascii', '.gen']
 
     while True:
         base, ext = os.path.splitext(base)
@@ -83,15 +87,43 @@ class EbookList(gtk.ListStore):
     """
     List of ebooks:
 
-        [(author, title, language, file_name), ...]
+        [(author, title, language, file_name, visit_timestamp), ...]
     """
-    
-    def __init__(self, search_dirs):
-        gtk.ListStore.__init__(self, str, str, str, str)
+
+    def __init__(self, search_dirs, recent_map=None):
+        gtk.ListStore.__init__(self, str, str, str, str, int)
         self.search_dirs = search_dirs
-    
+        if recent_map:
+            self.recent_map = dict(recent_map)
+        else:
+            self.recent_map = {}
+
+        def to_show(model, it):
+            return model.get(it, 4)[0] > 0
+
+        lst = self.filter_new()
+        lst.set_visible_func(to_show)
+        self.recent_list = gtk.TreeModelSort(lst)
+        self.recent_list.set_sort_column_id(4, gtk.SORT_DESCENDING)
+
+    def mark_visited(self, file_name, recent_map2=None):
+        stamp = int(time.time())
+        self.recent_map[file_name] = stamp
+        if recent_map2 is not None:
+            recent_map2[file_name] = stamp
+
+        # Update model
+        def walk(model, path, iter, user_data):
+            if model.get(iter, 3)[0] == file_name:
+                model.set_value(iter, 4, stamp)
+                return True
+            else:
+                return False
+        self.foreach(walk, None)
+
     def add(self, author=u"", title=u"", language=u"", file_name=""):
-        return self.append((author, title, language, file_name))
+        stamp = self.recent_map.get(file_name, -1)
+        return self.append((author, title, language, file_name, stamp))
 
     def delete_file(self, it):
         entry = self[it]
@@ -134,17 +166,22 @@ class EbookList(gtk.ListStore):
                         if m:
                             g = m.groupdict()
                             entry = (reformat_auth(g.get('auth', author_name)),
-                                     g.get('titl', base),
+                                     reformat_title(g.get('titl', base)),
                                      g.get('lang', ''),
-                                     full_path)
+                                     full_path,
+                                     self.recent_map.get(full_path, -1))
                             break
                     if entry is None:
-                        entry = (author_name, base, "", full_path)
+                        entry = (author_name, base, "", full_path,
+                                 self.recent_map.get(full_path, -1))
                     files.append(entry)
 
         def reformat_auth(auth):
             return auth.replace("; ", "\n")
 
+        def reformat_title(title):
+            return title.replace("; ", "\n")
+
         def really_add(r):
             for x in r:
                 self.append(x)
@@ -160,6 +197,11 @@ class EbookList(gtk.ListStore):
 
         start_thread(do_walk_tree, self.search_dirs)
 
+
+#------------------------------------------------------------------------------
+# Search results list
+#------------------------------------------------------------------------------
+
 NEXT_ID = -10
 
 class GutenbergSearchList(gtk.ListStore):
@@ -279,12 +321,11 @@ class GutenbergSearchList(gtk.ListStore):
             if isinstance(result, Exception):
                 callback(result)
                 return
-                
+
             r, infodict = result
             info.category = infodict['category']
-            for url, format, encoding, compression in r:
-                msg = [x for x in format, encoding, compression if x]
-                info.add(url, ', '.join(msg))
+            for url, description in r:
+                info.add(url, description)
             if callback:
                 callback(info)
 
@@ -311,15 +352,19 @@ def transpose_articles(text):
                 parts[0] = p.strip()
     return u"\n".join(parts).strip()
 
-def ellipsize(text, max_length=80):
+def ellipsize(text, max_length=80, ellipsis=u"..."):
     pieces = text.split(" ")
     size = -1
     for k, piece in enumerate(pieces):
-        if size + 1 + len(piece) + 4 > max_length:
+        if size + 1 + len(piece) + len(ellipsis) > max_length:
             if size > 0:
-                return u" ".join(pieces[:k]) + u" ..."
+                return u" ".join(pieces[:k]) + ellipsis
             else:
-                return pieces[0][:125] + u"..."
+                rem = max_length - len(ellipsis)
+                if rem >= 0:
+                    return pieces[0][:rem] + ellipsis
+                else:
+                    return ellipsis[:max_length]
         size += len(piece) + 1
     return u" ".join(pieces)
 
@@ -366,24 +411,33 @@ class DownloadInfo(gtk.ListStore):
                                  if not x.startswith('tr. ')])
         base_author = clean_filename(base_author)
 
+        title = self.title.replace("\n", "; ")
+
         url_base = url.split('/')[-1]
         try:
             ext = url_base.split('.', 1)[1]
         except IndexError:
             ext = ''
 
+        if ext == 'txt.utf8':
+            ext = 'txt'
+        elif ext == 'txt.ascii':
+            ext = 'txt'
+        elif ext == 'html.gen':
+            ext = 'html'
+
         if not ext and 'plucker' in format:
             ext = 'pdb'
             url_base += '.pdb'
 
         if self.author and self.title and self.language:
             base_name = u"%s - %s [%s]" % (author,
-                                           self.title,
+                                           title,
                                            self.language.lower())
         elif self.author and self.title:
-            base_name = u"%s - %s" % (author, self.title)
+            base_name = u"%s - %s" % (author, title)
         elif self.title:
-            base_name = u"%s" % self.title
+            base_name = u"%s" % title
         else:
             base_name = u"Etext %d" % self.etext_id
         
@@ -396,6 +450,8 @@ class DownloadInfo(gtk.ListStore):
         else:
             file_name = clean_filename(base_name)
 
+        file_name = trim_filename(file_name, max_length=255)
+
         if base_author:
             path = os.path.join(base_directory, base_author, file_name)
         else:
@@ -403,7 +459,7 @@ class DownloadInfo(gtk.ListStore):
 
         if os.path.isfile(path) and not overwrite:
             raise OverwriteFileException()
-        
+
         dir_path = os.path.dirname(path)
         if not os.path.isdir(dir_path):
             os.makedirs(dir_path)
@@ -432,6 +488,25 @@ class DownloadInfo(gtk.ListStore):
         
         run_in_background(do_download, url, callback=on_finish)
 
+def trim_filename(fn, max_length=255):
+    if len(fn) <= max_length:
+        return fn
+
+    m = re.match(ur'^(.*?)(\.[a-zA-Z0-9.]+)$', fn)
+    if m:
+        base = m.group(1)
+        ext = m.group(2)
+    else:
+        base = fn
+        ext = u""
+
+    max_base_length = max_length - len(ext)
+    if max_base_length < 0:
+        max_base_length = 0
+
+    base = ellipsize(fn, max_length=max_base_length, ellipsis=u"\u2026")
+    return base + ext
+
 def clean_filename(s):
     """
     Encode file name in filesystem charset and remove illegal characters
@@ -441,6 +516,7 @@ def clean_filename(s):
     s = re.sub(r'[\x00-\x1f"\*\\/:<>?|]', '', s)
     return s
 
+
 #------------------------------------------------------------------------------
 # Configuration backend
 #------------------------------------------------------------------------------
