@@ -87,7 +87,7 @@ class EbookText(gtk.TextBuffer):
         basefn, ext = os.path.splitext(filename)
         if ext in ('.html', '.htm'):
             self._load_html(f)
-        elif ext in ('.txt', '.rst'):
+        elif ext in ('.txt', '.rst', '.utf8', '.ascii'):
             self._load_plain_text(f)
         elif ext in ('.pdb'):
             self._load_plucker(f)
@@ -111,6 +111,7 @@ class EbookText(gtk.TextBuffer):
             para = u""
             omit = 0
             slurp_space = True
+            in_pre = False
 
             def handle_starttag(self, tag, attrs):
                 self.flush()
@@ -139,6 +140,8 @@ class EbookText(gtk.TextBuffer):
                     self.omit += 1
                 elif tag == 'meta' and not self.in_body:
                     self.handle_meta(attrs)
+                elif tag == 'pre':
+                    self.in_pre = True
                 elif tag == 'img':
                     attrs = dict(attrs)
                     if 'alt' in attrs and attrs['alt'].strip():
@@ -174,6 +177,8 @@ class EbookText(gtk.TextBuffer):
                 elif tag in ('i', 'em', 'b', 'strong', 'bold'):
                     if self.tags:
                         self.tags.pop()
+                elif tag == 'pre':
+                    self.in_pre = False
 
             def handle_data(self, data):
                 if self.omit:
@@ -185,11 +190,13 @@ class EbookText(gtk.TextBuffer):
                         data = unicode(data, 'latin1')
 
                 data = data.replace('\r', '')
-                data = data.replace('\n', ' ')
-                if self.slurp_space:
-                    data = data.lstrip()
-                self.para += data
-                self.slurp_space = False
+                if not self.in_pre:
+                    data = data.replace('\n', ' ')
+                    if self.slurp_space:
+                        data = data.lstrip()
+                if data:
+                    self.para += data
+                    self.slurp_space = False
 
             def handle_charref(self, name):
                 try:
@@ -280,6 +287,10 @@ class EbookText(gtk.TextBuffer):
 
         NS = "{http://www.gribuser.ru/xml/fictionbook/2.0}"
 
+        par_had_text = True
+        par_mark = self.create_mark('par', self.get_end_iter(),
+                                    left_gravity=True)
+
         tags = []
         text = []
         for event, elem in etree.iterparse(f, ['start', 'end']):
@@ -287,11 +298,25 @@ class EbookText(gtk.TextBuffer):
             if el_tag.startswith(NS):
                 el_tag = el_tag[len(NS):]
 
-            if el_tag == 'p' and event == 'start':
+            if el_tag in ('p', 'v') and event == 'start':
                 flush_text()
                 text.append(u"\n")
                 if elem.text:
                     text.append(elem.text.lstrip())
+                    par_had_text = True
+                else:
+                    par_had_text = False
+                    self.move_mark(par_mark, self.get_end_iter())
+            elif el_tag == 'p' and event == 'end':
+                if elem.text and not par_had_text:
+                    pos = self.get_iter_at_mark(par_mark)
+                    pos.forward_char()
+                    self.insert_with_tags(pos,
+                                          elem.text.encode('utf-8'),
+                                          *tags)
+                par_had_text = True
+            elif el_tag == 'stanza':
+                text.append(u"\n")
             elif el_tag in ('emphasis', 'strong', 'style', 'a'):
                 tag = {'emphasis': self.tag_emph,
                        'strong': self.tag_bold}.get(el_tag, None)
@@ -308,6 +333,8 @@ class EbookText(gtk.TextBuffer):
                         text.append(elem.tail)
                     if tag and tag in tags:
                         tags.remove(tag)
+            elif el_tag == 'section':
+                text.append(u"\n")
             elif el_tag == 'title':
                 if event == 'start':
                     flush_text()
